/***************************************************************************************
受信したシリアル信号からの指示でベルを鳴らすXBee子機の実験用サンプルです。

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/

#include <xbee.h>
#include <LiquidCrystal.h>
#include "pitches.h"
#define     PIN_BUZZER      2                           // Digital 2にスピーカを接続

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
byte buzzer=0x00;                                       // ブザー状態
char s[34]="Example 57 Bell";                           // 表示する文字列用の変数

void setup(){
    lcd.begin(16, 2);                                   // LCDの初期化
    lcd.clear(); lcd.print(s); s[17]='\0';              // タイトル表示
    xbee_init( 0x00 );                                  // XBeeの初期化
    xbee_atcb( 4 );                                     // ネットワーク初期化
    xbee_atnj(0);                                       // 孫機に対するジョイン拒否
    lcd.setCursor(0, 1);                                // 液晶の２行目に移動
    lcd.print("Commissioning");                         // 文字を表示
    while( xbee_atai() > 0x01 ){                        // ネットワーク参加状況を確認
        delay(3000);
        xbee_atcb(1);                                   // ネットワーク参加ボタンを押下
    }
    lcd.clear();
}

void loop(){
    byte i;
    byte len=0;                                         // 表示する文字の長さ
    XBEE_RESULT xbee_result;                            // 受信データ用

    byte data = xbee_rx_call( &xbee_result );
    if( xbee_result.MODE == MODE_UART){                 // UARTシリアルデータを受信
        if(xbee_result.DATA[0] == 0x1B){                // ESCコードが先頭だったとき
            if(xbee_result.DATA[1]==(byte)'B'){         // コマンドが「B」の時
                buzzer = xbee_result.DATA[2];           // ブザー値を設定
                lcd.clear(); lcd.print("Ringing");
            }
        }else{                                          // ESCコードでは無い時
            for(i=0;i<34;i++) s[i]='\0';                // 変数sを初期化
            for(i=0;i<data;i++){                        // 受信したテキストをsに代入
                if(isprint((char)xbee_result.DATA[i])){ // 受信文字が表示可能の時
                    s[len] = (char)xbee_result.DATA[i]; // 表示用の変数に代入
                    len++;                              // 文字数のインクリメント
                    if( len == 16) len=17;              // 2行目はs[17]～
                }
                if( len >= 33 ) break;                  // 32文字目でループを抜ける
            }
        }
    } else if( xbee_result.MODE ){                      // その他の受信時
        lcd.clear();
        lcd.print( "MODE="); lcd.print(xbee_result.MODE,  HEX);
        lcd.print(" STAT="); lcd.print(xbee_result.STATUS,HEX);
        lcd.setCursor(0,1);
        for(i=0;i<8;i++){                               // 送信元のアドレスを表示
            lcd.print(xbee_result.FROM[i]>>4, HEX);
            lcd.print(xbee_result.FROM[i]%16, HEX);
        }
    }
    if(buzzer){
        tone(PIN_BUZZER,NOTE_A7,100);                   // ブザーに音階A7を出力
        delay(200);
        buzzer--;
        if(buzzer==0) len=0xFF;                         // ブザーの終了
    }
    if(len){                                            // 表示する文字が存在する時
        lcd.clear();
        lcd.print( s );                                 // 受信したテキストを表示
        lcd.setCursor(0,1);
        lcd.print( &s[17] );                            // 受信したテキストs[17]～
    }
}
