/***************************************************************************************
ガスセンサの測定値をXBee親機に送信するXBee子機の実験用サンプルです。
メタンガスセンサ    MQ-4    RL=22kΩ

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/
#include <xbee.h>
#include <LiquidCrystal.h>
#include "pitches.h"

#define     PIN_BUZZER      2                   // Digital 2にスピーカを接続
#define     PIN_CH4         1                   // Analog 1にメタンガスセンサ
#define     CH4_Ro          10                  // メタン1000ppm時のセンサ抵抗値[kΩ]

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
byte buzzer=0x00;                                   // ブザー状態
byte dev[]={0,0,0,0,0,0,0,0};                       // XBee親機アドレス（全てゼロ）
byte trig=0;                                        // 測定待ち用
int val;                                            // CH4ガスの読み取り値
float ch4;                                          // CH4ガスの測定値

void setup(){
    lcd.begin(16, 2);                               // LCDのサイズを16文字×2桁に設定
    lcd.clear();                                    // 画面消去
    lcd.print("Example 59 Gas");                    // 文字を表示
    xbee_init( 0x00 );                              // XBee用COMポートの初期化
    xbee_atcb(4);                                   // ネットワーク初期化
    xbee_atnj(0);                                   // 孫機に対するジョイン拒否
    lcd.setCursor(0, 1);                            // 液晶の２行目に移動
    lcd.print("Commissioning");                     // 文字を表示
    while( xbee_atai() > 0x01 ){                    // ネットワーク参加状況を確認
        for(int i=0;i<6;i++){
            tone(PIN_BUZZER,NOTE_A7,50);            // ペアリング中を表す音を鳴らす
            delay(500);
        }
        xbee_atcb(1);                               // ネットワーク参加ボタンを押下
    }
    lcd.clear();
}

void loop(){
    byte i,j;
    char s[40];                                     // 送信用の文字列
    XBEE_RESULT xbee_result;                        // 受信データ(詳細)用の変数の定義
    
    if( trig == 0){
        val = analogRead(PIN_CH4);                  // メタンガスセンサ読み取り
        ch4 = pow(10,3.0-2.7*log10(22*(1023/(float)val-1)/CH4_Ro));
        lcd.clear();                                // 液晶の画面消去
        lcd.print("CH4 = ");                        // 液晶に文字を表示
        if( ch4 < 10 ) lcd.print( ch4 , 2);         // 変数ch4の値を表示
        else lcd.print( ch4 , 0);
        lcd.print("ppm");                           // 変数ch4の値を表示
        sprintf(s, "%05d\r", (int)ch4 );            // XBee送信用データ作成
        xbee_uart(dev,s);                           // データを送信
        if(ch4 > 1000) buzzer=1;
        else if(ch4 > 100) tone(PIN_BUZZER,NOTE_A7,100);
    }
    trig--;                                         // 自動送信しない場合はこの行を削除

    xbee_rx_call( &xbee_result );
    if( xbee_result.MODE == MODE_UART ){            // シリアルデータを受信したとき
        if(xbee_result.DATA[0] == 0x1B){            // ESCコードが先頭だったとき
            for(i=0;i<16;i++)dev[i]=xbee_result.FROM[i];    // 送信元をdevに保存
            if(xbee_result.DATA[1]=='B'){           // コマンドが'B'の時
                buzzer=1;                           // 警報音を鳴らす
                if(xbee_result.DATA[2]==0x00){      // 値が0x00の時
                    buzzer=0;                       // 警報音を止める
                }
            }
            trig=0;                                 // 次回のloopで測定して送信
        }
    }

    /* ブザー */
    if( buzzer ){                                   // buzzerが0では無い時に音を鳴らす
        if(buzzer==1){
            tone(PIN_BUZZER,NOTE_B7,200);
            lcd.setCursor(0, 1);
            lcd.print("Gas is Leaking! ");
        }else if(buzzer==31){
            tone(PIN_BUZZER,NOTE_G7,200);
            lcd.setCursor(0, 1);
            lcd.print("               ");
        }else if(buzzer>60) buzzer=0;
        buzzer++;
    }
}
