/***************************************************************************************
受信したテキストやレベルをキャラクタ液晶に表示します。孫機の管理と制御も行います。

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/
#include <xbee.h>
#include "pitches.h"

#define ADAFRUIT   // ← 使用する液晶が DF ROBOT LCD Keypad Shieldの場合は本行を削除する

#ifndef ADAFRUIT
    // DF ROBOT LCD Keypad Shieldの場合
    #include <LiquidCrystalDFR.h>
    LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
#else
    // Adafruit I2C接続LCDの場合(Adafruit_RGBLCDShieldライブラリが必要)
    #include <Wire.h>
    #include <Adafruit_MCP23017.h>
    #include <Adafruit_RGBLCDShield.h>
    Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();
#endif
//#define PIN_BUZZER  2                                 // Digital 2にスピーカを接続

const byte coord[]= {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};  // 親機
byte dev_gpin[]   = {0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF};  // 孫機
byte dev[]        = {0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF};  // 汎用
byte counter;
byte menu_n = 0;
const char menu[10][11]={"Pairing ON","Force Shot","Repeat ON ","Repeat OFF",
        "Disp=Level","Disp=Char ","TxRepo ON ","TxRepo OFF","Ope PAN ID","Reset NetW"};
char s[17] = "by Wataru Kunino";
char tx[17];
boolean force_repeat = 0;                               // 測定の繰り返し 0=OFF／1=ON
boolean tx_rep = 0;                                     // 測定結果の報告 0=OFF／1=ON
boolean disp_lv_chr = 1;                                // 結果表示 0=レベル／1=数値表示
boolean dev_gpin_en = 0;                                // 孫機とのペアリング状況
unsigned char font_lv[4][8]={
    {0b00000,0b10000,0b00000,0b10000,0b00000,0b10000,0b00000,0b10101},
    {0b11000,0b11000,0b11000,0b11000,0b11000,0b11000,0b11000,0b11101},
    {0b11011,0b11011,0b11011,0b11011,0b11011,0b11011,0b11011,0b11011},
    {0b00000,0b10000,0b00000,0b10000,0b00000,0b10000,0b00000,0b10000}
};

void lcd_cls( const byte line ){                        // 指定した行を消去する関数
    lcd.setCursor(0,line);
    for(byte i=0;i<16;i++)lcd.print(" ");
    lcd.setCursor(0,line);
}

void lcd_print_hex(const byte in){                      // 16進数2桁(1バイト)の表示関数
    lcd.print( in>>4 , HEX );
    lcd.print( in%16 , HEX );
}

void lcd_print_level(const byte lev1, const byte lev2){ // レベルメータ表示を行う関数
    byte i;                                         
    lcd.home();
    for(i=0 ; i < 5 ; i++){
        if( lev1 <= 2 * i )             lcd.write( (byte)0x00 );
        else if( lev1 - 2 * i == 1 )    lcd.write( (byte)0x01 );
        else                            lcd.write( (byte)0x02 );
    }
    if(lev1>10) lcd.print("F"); else lcd.write(3);
    lcd.print(" ");
    lcd.setCursor(7,0);
    for(i=0 ; i < 5 ; i++){
        if( lev2 <= 2 * i )             lcd.write( (byte)0x00 );
        else if( lev2 - 2 * i == 1 )    lcd.write( (byte)0x01 );
        else                            lcd.write( (byte)0x02 );
    }
    if(lev2>10) lcd.print("F"); else lcd.write(3);
}

void lcd_print_results( XBEE_RESULT *xbee_result ){     // 測定結果の表示と送信
    byte i;
    byte lev1 = 0xFF, lev2 = 0xFF;
    char buf[8];
    tx[0]='\0';                                         // 送信バッファの消去
    for(i=1;i<=3;i++){
        if( xbee_result->ADCIN[i] < 1024 ){             // アナログ入力有効時
            sprintf(buf,"%d,",xbee_result->ADCIN[i]);   // アナログ値を保存
        }else{
            sprintf(buf,"%1d,",((xbee_result->GPI.BYTE[0])>>i)&1 );
        }                                               // デジタル時の保存
        strcat(tx,buf);                                 // 送信バッファへ追加
    }
    tx[strlen(tx)-1]='\0';                              // 末尾のコンマを消去
    if(disp_lv_chr){                                    // レベル表示の時
        for(i=1;i<=3;i++){
            if( xbee_result->ADCIN[i] < 1024 ){         // アナログ時のレベル設定
                if(     lev1==0xFF)lev1=(byte)((xbee_result->ADCIN[i])/100);
                else if(lev2==0xFF)lev2=(byte)((xbee_result->ADCIN[i])/100);
            }
        }
        if(lev1==0xFF){                                 // デジタル入力時のレベル設定
            lev1 = 10*(((xbee_result->GPI.BYTE[0])>>1)&1);
            lev2 = 10*(((xbee_result->GPI.BYTE[0])>>2)&1);
        }else if(lev2==0xFF){
            lev2 = 10*(((xbee_result->GPI.BYTE[0])>>1)&1);
        }
        lcd_print_level(lev1,lev2);                     // レベル表示の実行
    }else{
        lcd_cls(0);
        lcd.print(tx);                                  // 数値表示の実行
    }
    if(tx_rep){
        strcat(tx,"\r");
        xbee_uart(coord,tx);                            // 親機XBeeへ結果を送信
    }
}

void lcd_print_address(char *str, byte *dev ){          // アドレス表示用の関数
    byte i;
    byte start = 4;                                     // アドレスの表示開始バイト
    lcd_cls(1);                                         // 2行目を消去
    if( str[0] == '\0' ) start=0;                       // strに文字なし時は8バイト表示
    else lcd.print( str );                              // 文字あり時は文字+4バイト表示
    for(i=start;i<8;i++) lcd_print_hex(dev[i]);         // アドレスを液晶に表示する
}

void setup(){
    lcd.begin(16, 2);                                   // LCDの初期化
    lcd.clear(); lcd.print("M2M Display   --");         // 起動時の表示
    lcd.setCursor(0,1); lcd.print(s);
    xbee_init( 0x00 );                                  // XBeeの初期化
    xbee_atnj(0);                                       // ジョイン拒否
    for(byte i=0;i<4;i++) lcd.createChar(i,&(font_lv[i][0]));   // レベルフォントの代入
    switch( xbee_atvr() ){                              // XBeeのZigBee種別を取得
        case ZB_TYPE_COORD:  sprintf(s,"ZigB COORDINATOR"); break;
        case ZB_TYPE_ROUTER: sprintf(s,"ZigBee Router   "); break;
    }
}

void loop(){
    byte i;
    byte data, buttons;                                 // XBee、LCD Padからの入力用
    XBEE_RESULT xbee_result;                            // 受信データ用

    buttons = lcd.readButtons();                        // Keypadからの入力
    if( counter >= 200 ){                               // 約2秒ごとの処理
        lcd.setCursor(14,0);
        lcd_print_hex(xbee_atai());                     // XBeeの状態表示（右上に常時）
        lcd.setCursor(0,1);
        lcd.print(s);                                   // 液晶へのメッセージ表示(常時)
        if(force_repeat) xbee_force(dev_gpin);          // 孫機へデータ取得指示の送信
        counter = 0;                                    // 本処理用のカウンタのリセット
    }
    counter++;
    if( buttons != 0 ){
        counter = 0;
        lcd_cls(1);                                     // 文字表示行の消去
        switch( buttons ){                              // ボタンに応じて
            case BUTTON_UP:                             // 上ボタンが押された時
                lcd.print("Commisioning");
                xbee_atcb(1);                           // 親機へのジョイン開始
                delay(3000);
                if( xbee_ping( coord ) != 0xFF){        // PING確認
                    xbee_from( dev );                   // PING応答元のアドレスを取得
                    lcd_print_address("COORD. =",dev);
                }else lcd.print("Fail");
                break;
            case BUTTON_DOWN:                           // 下ボタンが押された時
                lcd.print("Enabled Net Join");
                xbee_atcb(2);                           // 孫機のジョイン許可
                break;
            case BUTTON_RIGHT:                          // 左ボタンでメニュー番号を+1
                if( menu_n < 9 ) menu_n++; else menu_n = 0;
                lcd.print( menu[menu_n] );
                break;
            case BUTTON_LEFT:                           // 左ボタンでメニュー番号を-1
                if( menu_n > 0 ) menu_n--; else menu_n = 9;
                lcd.print( menu[menu_n] );
                break;
            case BUTTON_SELECT:                         // セレクトボタンでメニュー実行
                lcd.print( menu[menu_n] ); lcd.print("> DONE");
                switch( menu_n ){
                    case 0: xbee_atcb(2);               // "Pairing ON" 孫機とペアリング
                        dev_gpin_en=0;
                        break;
                    case 1: xbee_force( dev_gpin );     // "Force Shot" データ要求
                        break;
                    case 2: force_repeat=1;             // "Repeat ON " データ連続要求
                        break;
                    case 3: force_repeat=0;             // "Repeat OFF" 連続要求の停止
                        break;
                    case 4: disp_lv_chr=1;              // "Disp=Level" メーター表示
                        break;
                    case 5: disp_lv_chr=0;              // "Disp=Char " 数値表示
                        break;
                    case 6: tx_rep=1;                   // "TxRepo ON " 結果情報の送信
                        break;
                    case 7: tx_rep=0;                   // "TxRepo OFF" 情報送信の停止
                        break;
                    case 8: xbee_atop( dev );           // "Ope PAN ID" PAN ID取得
                        lcd_print_address("",dev);
                        break;
                    case 9: xbee_atcb(4);               // "Reset NetW" ZBネットリセット
                        delay(1000);
                        setup();
                        break;
                }
                break;
        }
        while( lcd.readButtons() ) delay(100);          // キー押下の開放待ち
        delay(500);
    }

    /* データ受信(待ち受けて受信する) */
    data = xbee_rx_call( &xbee_result );
    if( xbee_result.MODE ){ // 何かを受信した
        counter = 0;
        switch( xbee_result.MODE ){                     // 受信したデータの内容に応じて
            case MODE_UART:                             // シリアルデータを受信
                lcd_cls(1);                             // 文字表示行の消去
                if( xbee_result.DATA[0] == 0x1B ){      // ESCコードが先頭だったとき
                    switch( xbee_result.DATA[1]  ){ 
                        case (byte)'L':
                            for(i=0;i<2;i++)            // 11よりも大きい場合は0に変更
                                if(xbee_result.DATA[i+2]>11)xbee_result.DATA[i+2]=0;
                            lcd_print_level(xbee_result.DATA[2],xbee_result.DATA[3]);
                            break;
                    //  case (byte)'B':
                    //      tone(PIN_BUZZER,NOTE_A7,100);   // ブザーに音階A7を出力
                    //      break;
                        default:
                            sprintf(s,"ESC(0x%02X)=0x%02X",
                                xbee_result.DATA[1],xbee_result.DATA[2]);
                            break;
                    }
                }else{                                  // テキスト文字データを表示
                    for(i=0;i<=16;i++) s[i]=' ';        // 文字列変数sの初期化
                    if(data > 16) data=16;              // 文字の扱いは16文字まで
                    for(i=0;i<data;i++){                // 受信データを文字列変数sへ代入
                        if(isprint((char)xbee_result.DATA[i]))
                            s[i]=(char)xbee_result.DATA[i];
                    }
                }
                counter = 255;
                break;
            case MODE_GPIN:                             // 孫機のGPIOからのデータを受信
                lcd_print_results(&xbee_result);
                lcd_print_address("Rx GPIN ",xbee_result.FROM);
                counter = 0;
                break;
            case MODE_RESP:                             // リモートATコマンド応答を受信
                lcd_print_results(&xbee_result);
                lcd_print_address("Rx RESP ",xbee_result.FROM);
                counter = 0;
                break;
            case MODE_IDNT:                             // 孫機デバイスを発見
                if(dev_gpin_en) lcd_print_address("Rx IDNT ",dev_gpin);
                else{
                    dev_gpin_en=1;
                    for(i=0;i<8;i++) dev_gpin[i]=xbee_result.FROM[i];
                    xbee_ratd_myaddress(dev_gpin);      // 本機アドレスを孫機に設定
                    lcd_print_address("Child  =",dev_gpin);
                }
                counter = 0;
                break;
            default:                                    // その他のパケット受信時
                lcd_cls(1);        lcd.print("MODE=");lcd_print_hex(xbee_result.MODE);
                lcd.setCursor(8,1);lcd.print("STAT=");lcd_print_hex(xbee_result.STATUS);
                counter = 0;
                break;
        }
    }
}
