/***************************************************************************************
Bluetoothモジュール RN-42XVPのGPIOを制御してRSSI LED(XBee 6ピン)を点滅します。

                                                Copyright (c) 2013 Wataru KUNINO
                                                http://www.geocities.jp/bokunimowakaru/
***************************************************************************************/

#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

/* DF ROBOT 製液晶を使用する場合は上記を下記に入れ替えてください。
    #include <LiquidCrystalDFR.h>
    LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
*/

/* シリアルの受信バッファを消去する関数 */
void bt_rx_clear(){
    while( Serial.available() ){                // 受信データが残っている場合
        Serial.read();                          // 空読み(消去)
        delay(2);
    }   
}

/* コマンドの送信を行う関数 */
void bt_tx(char *cmd){
    bt_rx_clear();                              // シリアルの受信バッファを消去する
    Serial.println( cmd );                      // コマンドの送信
    Serial.flush();                             // 送信完了待ち
    bt_rx_clear();                              // シリアルの受信バッファを消去する
}

/* コマンドモードに入るための送信を行う関数 */
void bt_tx_mode(void){
    bt_rx_clear();                              // シリアルの受信バッファを消去する
    Serial.print("###");                        // コマンドモードに入る命令を実行
    delay(100);                                 // 応答待ち
    if( !Serial.available() ){
        Serial.print("$$$");                    // コマンドモードに入る命令を実行
        Serial.flush();                         // 送信完了待ち
    }
    bt_rx_clear();                              // シリアルの受信バッファを消去する
}

void setup(){
    lcd.begin(16, 2);
    lcd.print("Bluetooth RSSI  ");              // タイトル文字を表示
    Serial.begin(115200);                       // シリアルポートの初期化
}

void loop(){
    /* コマンドモードへの移行処理 */
    lcd.setCursor(0,1);
    lcd.print("Command Mode");
    bt_tx_mode();                               // コマンドモードへの移行
    
    /* RSSI LEDの点滅処理 */
    bt_tx("S@,4040");                           // GPIOポート6を出力に設定}
    for(int i=0; i<5; i++ ){
        bt_tx("S&,4040");                       // GPIOポート6へHighレベルを出力
        delay(1000);                            // 待ち時間 1秒
        bt_tx("S&,4000");                       // GPIOポート6へLowレベルを出力
        delay(1000);                            // 待ち時間 1秒
    }
    
    /* コマンドモードの解除処理 */
    bt_tx("---");                               // コマンドモード解除
    lcd.clear();
    lcd.print("Disconnected");
    lcd.setCursor(0,1);
    lcd.print("Hit Any Key");
    while( lcd.readButtons()==0 );              // キー入力待ち
    lcd.clear();
}
